import socket
import threading
import tkinter as tk
from tkinter import scrolledtext
import hashlib

class ChatClient:
    def __init__(self, root):
        self.root = root
        self.root.title("Chat Client")

        self.frame = tk.Frame(self.root)
        self.frame.pack(padx=10, pady=10)

        self.text_area = scrolledtext.ScrolledText(self.frame, state='disabled', width=50, height=20)
        self.text_area.grid(row=0, column=0, columnspan=3)

        self.message_entry = tk.Entry(self.frame, width=40)
        self.message_entry.grid(row=1, column=0)

        self.send_button = tk.Button(self.frame, text="Send", command=self.send_message)
        self.send_button.grid(row=1, column=1)

        self.smiley_button = tk.Button(self.frame, text="Smiley", command=self.open_smiley_menu)
        self.smiley_button.grid(row=1, column=2)

        self.disconnect_button = tk.Button(self.frame, text="Disconnect", command=self.disconnect)
        self.disconnect_button.grid(row=2, column=2, pady=5)

        self.server_ip = tk.StringVar()
        self.username = tk.StringVar()
        self.password = tk.StringVar()

        self.server_ip.set("Server-IP")
        self.username.set("Username")
        self.password.set("Password")

        self.server_ip_entry = tk.Entry(self.frame, textvariable=self.server_ip, width=30)
        self.server_ip_entry.grid(row=3, column=0, columnspan=2, sticky='w')
        self.server_ip_entry.bind("<FocusIn>", self.clear_placeholder_ip)
        self.server_ip_entry.bind("<FocusOut>", self.set_placeholder_ip)

        self.username_entry = tk.Entry(self.frame, textvariable=self.username, width=30)
        self.username_entry.grid(row=4, column=0, columnspan=2, sticky='w')
        self.username_entry.bind("<FocusIn>", self.clear_placeholder_username)
        self.username_entry.bind("<FocusOut>", self.set_placeholder_username)

        self.password_entry = tk.Entry(self.frame, textvariable=self.password, width=30, show="*")
        self.password_entry.grid(row=5, column=0, columnspan=2, sticky='w')
        self.password_entry.bind("<FocusIn>", self.clear_placeholder_password)
        self.password_entry.bind("<FocusOut>", self.set_placeholder_password)

        self.connect_button = tk.Button(self.frame, text="Connect", command=self.connect)
        self.connect_button.grid(row=6, column=0, columnspan=2, pady=5)

        self.register_button = tk.Button(self.frame, text="Register", command=self.register)
        self.register_button.grid(row=7, column=0, columnspan=2, pady=5)

        self.client_socket = None
        self.connected = False
        self.username_set = False

        self.message_entry.bind("<Return>", lambda event: self.send_message())

        # Dictionary for smiley options
        self.smileys = {
            "😊": ":)", "😢": ":(", "😃": ":D", "😋": ":P", "😲": ":O",
            "😉": ";)", "😂": ":LOL:", "😍": ":heart:", "😎": ":cool:",
            "😅": ":sweat:", "😭": ":cry:", "😠": ":angry:", "🤔": ":thinking:",
            "😜": ":wink:", "😏": ":smirk:", "😇": ":angel:", "🤗": ":hug:",
            "🤩": ":starstruck:", "🤯": ":mindblown:", "🥳": ":partying:", "😴": ":sleepy:",
            "🤤": ":drooling:", "🤡": ":clown:", "🥺": ":pleading:", "😈": ":devil:"
        }

        # Smiley dropdown menu
        self.smiley_menu = tk.Toplevel(self.root)
        self.smiley_menu.title("Select Emoji")
        self.smiley_menu.geometry("400x400")
        self.smiley_menu.withdraw()  # Initially hidden

        self.smiley_frame = tk.Frame(self.smiley_menu)
        self.smiley_frame.pack(fill='both', expand=True)

        self.smiley_canvas = tk.Canvas(self.smiley_frame)
        self.smiley_canvas.pack(side='left', fill='both', expand=True)

        self.smiley_scrollbar = tk.Scrollbar(self.smiley_frame, orient='vertical', command=self.smiley_canvas.yview)
        self.smiley_scrollbar.pack(side='right', fill='y')

        self.smiley_canvas.configure(yscrollcommand=self.smiley_scrollbar.set)

        self.smiley_container = tk.Frame(self.smiley_canvas)
        self.smiley_canvas.create_window((0, 0), window=self.smiley_container, anchor='nw')

        self.populate_smiley_menu()

        # Bind event to hide smiley menu
        self.smiley_menu.protocol("WM_DELETE_WINDOW", self.close_smiley_menu)

    def connect(self):
        if self.connected:
            self.text_area.config(state='normal')
            self.text_area.insert(tk.END, "[INFO] Already connected.\n")
            self.text_area.config(state='disabled')
            return

        server_ip = self.server_ip.get()
        username = self.username.get()
        password = self.password.get()
        hashed_password = hashlib.sha256(password.encode()).hexdigest()

        self.client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self.client_socket.connect((server_ip, 5555))
            self.client_socket.send(f"LOGIN {username} {hashed_password}".encode())
            response = self.client_socket.recv(1024).decode()
            if response == "LOGIN_SUCCESS":
                self.connected = True
                self.username_set = True
                self.text_area.config(state='normal')
                self.text_area.insert(tk.END, "[INFO] Connected! You can start chatting.\n")
                self.text_area.config(state='disabled')
                self.text_area.config(state='disabled')  # Set text_area to read-only
                threading.Thread(target=self.receive_messages, daemon=True).start()
            else:
                self.text_area.config(state='normal')
                self.text_area.insert(tk.END, "[ERROR] Login failed.\n")
                self.text_area.config(state='disabled')
        except Exception as e:
            self.text_area.config(state='normal')
            self.text_area.insert(tk.END, f"[ERROR] {e}\n")
            self.text_area.config(state='disabled')

    def register(self):
        server_ip = self.server_ip.get()
        username = self.username.get()
        password = self.password.get()
        hashed_password = hashlib.sha256(password.encode()).hexdigest()

        self.client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self.client_socket.connect((server_ip, 5555))
            self.client_socket.send(f"REGISTER {username} {hashed_password}".encode())
            response = self.client_socket.recv(1024).decode()
            self.text_area.config(state='normal')
            self.text_area.insert(tk.END, f"[INFO] {response}\n")
            self.text_area.config(state='disabled')
        except Exception as e:
            self.text_area.config(state='normal')
            self.text_area.insert(tk.END, f"[ERROR] {e}\n")
            self.text_area.config(state='disabled')

    def send_message(self):
        if self.client_socket and self.connected:
            message = self.message_entry.get()
            if message:
                # Insert message into the text area and send it
                self.text_area.config(state='normal')
                if not self.text_area.get("end-2c linestart", "end-1c").startswith(f"You: {message}\n"):
                    self.text_area.insert(tk.END, f"you: {message}\n")
                self.text_area.config(state='disabled')
                self.client_socket.send(f"MESSAGE {message}".encode())
                self.message_entry.delete(0, tk.END)

    def receive_messages(self):
        while True:
            try:
                message = self.client_socket.recv(1024).decode()
                if message:
                    # Display messages from other users only
                    if not message.startswith(f"{self.username.get()}:"):
                        self.text_area.config(state='normal')
                        self.text_area.insert(tk.END, f"{message}\n")
                        self.text_area.config(state='disabled')
            except:
                break

    def open_smiley_menu(self):
        self.smiley_menu.deiconify()  # Show the smiley menu

    def close_smiley_menu(self):
        self.smiley_menu.withdraw()  # Hide the smiley menu instead of destroying it

    def populate_smiley_menu(self):
        row, col = 0, 0
        for emoji, code in self.smileys.items():
            button = tk.Button(self.smiley_container, text=emoji, font=("Arial", 20), command=lambda e=emoji: self.add_smiley(e))
            button.grid(row=row, column=col, padx=5, pady=5)
            col += 1
            if col > 5:
                col = 0
                row += 1

        # Update canvas scroll region
        self.smiley_container.update_idletasks()
        self.smiley_canvas.config(scrollregion=self.smiley_canvas.bbox("all"))

    def add_smiley(self, smiley):
        current_text = self.message_entry.get()
        self.message_entry.delete(0, tk.END)
        self.message_entry.insert(tk.END, current_text + smiley)

    def disconnect(self):
        if self.client_socket:
            self.client_socket.close()
            self.connected = False
            self.username_set = False
            self.text_area.config(state='normal')
            self.text_area.insert(tk.END, "[INFO] Disconnected.\n")
            self.text_area.config(state='disabled')

    # Functions to handle focus on input fields
    def clear_placeholder_ip(self, event):
        if self.server_ip.get() == "Server-IP":
            self.server_ip.set("")

    def set_placeholder_ip(self, event):
        if not self.server_ip.get():
            self.server_ip.set("Server-IP")

    def clear_placeholder_username(self, event):
        if self.username.get() == "Username":
            self.username.set("")

    def set_placeholder_username(self, event):
        if not self.username.get():
            self.username.set("Username")

    def clear_placeholder_password(self, event):
        if self.password.get() == "Password":
            self.password.set("")

    def set_placeholder_password(self, event):
        if not self.password.get():
            self.password.set("Password")

if __name__ == "__main__":
    root = tk.Tk()
    client = ChatClient(root)
    root.mainloop()